#____________________________________________________________________________________
#
#    PersistentThreads (see README.txt for more information)
#    Copyright (C) 2002 Christoph Becker-Freyseng
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#    christoph@mmc-startup.com
#    http://www.beyond-thoughts.com
#    http://www.mmc-startup.com
#____________________________________________________________________________________
#



__doc__ = """PPHandler module.  This module makes an HTTP-call to it's 
	    own Zope-Server to trigger PP.process. This seems to be much more
	    stable than other aproaches but there may be problems 
	    with PP.process when it takes extremely long -> http-timeout.
	    There's is another severe Problem making ViaZope preferable:
	    When there's heavy load something in Zope/OS fails and
	    I get a "BrokenPipe" somewhere in medusa."""
__version__ = '0.1.1'  

from PConstants import ppHandlerURL, ppHandler_label, ppHandler_action, ppHandler_permission
from PTTools import checkPermission
import PPManager

import ZPublisher.Client

import random
import threading
import sys
import traceback



_accessKeysDict= {}
_manipulatorDict= {}
_ppTransferLock= threading.RLock()



def handlePPCall(ptID, userP, passwdP, maniListP=None):
    if ((maniListP is not None) and (not maniListP)):	# The List of Manipulators is an empty list
	return -2	# Just say manipulation complete
    _ppTransferLock.acquire()	# Add an access-key
    keyD= str(random.random())
    while _accessKeysDict.has_key(keyD):
	keyD= str(random.random())
    valueD= str(random.random())
    _accessKeysDict[keyD]= valueD
    if maniListP:
	_manipulatorDict[ptID]= maniListP	# We can just write here (no checking if overwriting) because under no circumstances this should be called a second time before the first call is finished
    _ppTransferLock.release()
    try:
	resD= ZPublisher.Client.call(ppHandlerURL, username=userP, password=passwdP, pThreadID=ptID, accKey=keyD, accValue=valueD)
	wtD= float(str(resD[1]))   # not the message, just the body of resD
    except:     # something went wrong, maybe ConflictError, so just wait a little and try again
	wtD= (random.random()*5) + 0.5
	sys.stderr.write('PThreads: triggering PPHandler this error occurred: ')	# Log this error
	traceback.print_exc(file=sys.stderr)
    _ppTransferLock.acquire()	# Remove this key
    if _accessKeysDict.has_key(keyD):
	del _accessKeysDict[keyD]
    if _manipulatorDict.has_key(ptID):
	del _manipulatorDict[ptID]
    _ppTransferLock.release()
    return wtD




def initPPHandler(prodP):
    try:
        ppHandlerD= getattr(prodP, ppHandler_action, None)
    except:
        ppHandlerD= None
    if ( (ppHandlerD is None) or (ppHandlerD != ppHandlerMethod) ):
        setattr(prodP, ppHandler_action, ppHandlerMethod)
	listD= []
	for eleR in prodP.manage_options:
	    if eleR.get('label', None) != ppHandler_label:
		listD.append(eleR)
	listD.append({'label':ppHandler_label, 'action':ppHandler_action,})
	prodP.manage_options= tuple(listD)
	prodP._p_changed= 1
	get_transaction().note('Added a "'+str(ppHandler_action)+'" to the "'+str(prodP)+'" Product')
	get_transaction().commit()



def ppHandlerMethod(self, pThreadID=None, accKey=None, accValue=None, REQUEST=None):
    """ handle a request to invoke a PP """
    pChkD= checkPermission(ppHandler_permission, self, REQUEST)		# Whaaa, Zope just didn't make it right
    if (pChkD is not None):
	return str(pChkD)
    if ((pThreadID is None) and (accKey is None) and (accValue is None) and (REQUEST is not None)):
	return 'PThreads-HTTP: The user "' + str(REQUEST.get('AUTHENTICATED_USER', '')) + '" can access the ppHandlerTrigger (see PersistentThreads-Help for more info).\n'
    pThreadID= str(pThreadID)
    accKey= str(accKey)
    accValue= str(accValue)
    _ppTransferLock.acquire()	# Do the Security Check
    if not _accessKeysDict.has_key(accKey):
        return str(-1)
    if (_accessKeysDict[accKey] != accValue):
        return str(-1)
    maniList= _manipulatorDict.get(pThreadID, None)
    _ppTransferLock.release()
    ppManD= PPManager._getPPMan(self)
    if ppManD is None:	# Oooops it's too early
	return '13.3'
    ppD= ppManD.getPP(pThreadID)
    if ppD is None:
	return '-1'
    if maniList:
	try:
	    for maniR in maniList:
		if not maniR._completed:
		    maniR.manipulate(ppD, REQUEST=REQUEST)
		    maniR._completed= 1
	    resD= -2
	except:
	    resD= 0	# Manipulation not completed
    else:
	resD= ppD.process(REQUEST=REQUEST)
    try:
	retD= float(resD)
    except:
	retD= -1
    if ((retD == -1) and ppD.autoBurry()):
	ppManD.delPP(pThreadID)
    return str(retD)





