#____________________________________________________________________________________
#
#    PersistentThreads (see README.txt for more information)
#    Copyright (C) 2002 Christoph Becker-Freyseng
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#    christoph@mmc-startup.com
#    http://www.beyond-thoughts.com
#    http://www.mmc-startup.com
#____________________________________________________________________________________
#



__doc__ = """PPHandler module. This PPHandler works via the Zope-architecture
		for creating new object-spaces for a new thread. This whole
		stuff should work automagically via ZPublisher.Zope. 
		However there seem to be problems deep down in Zope's 
		threading-architecture. You might get errors like
		"None has no attribute load/tpc_begin".
		I wasn't able to figure out the reason for this since a
		long time. An internet-search shows that even the
		DC-people aren't sure about it.
		If you habe trouble with this module use another more stable 
		one. The advantage of this one is that there is no overhead
		for http-connections...
		
		If you know the reason for the threading-problems esp.
		using ZPublisher.Zope in a newly created thread
		PLEASE tell me christoph@mmc-startup.com . Thank You 
		
		I think I solved those Problems. I'll post a detailed report
		somewhere sometimes. If you're interested now drop a an email for 
		some short chunks."""
__version__ = '0.1.1'  

from PConstants import ppHandlerLocation, ppHandler_label, ppHandler_action, ppHandler_permission
from PTTools import checkPermission
import PPManager

import random
import threading
import sys
import traceback

import ZPublisher	# Not 100% sure if there are no problems when importing the ZPublisher in an other Thread than I'll use it later
import AccessControl.SecurityManagement	# Whaaaa see below



_accessKeysDict= {}
_manipulatorDict= {}
_ppTransferLock= threading.RLock()



def handlePPCall(ptID, userP, passwdP, maniListP=None):
    if ((maniListP is not None) and (not maniListP)):	# The List of Manipulators is an empty list
	return -2	# Just say manipulation complete
    _ppTransferLock.acquire()	# Add an access-key
    keyD= str(random.random())
    while _accessKeysDict.has_key(keyD):
	keyD= str(random.random())
    valueD= str(random.random())
    _accessKeysDict[keyD]= valueD
    if maniListP:
	_manipulatorDict[ptID]= maniListP	# We can just write here (no checking if overwriting) because under no circumstances this should be called a second time before the first call is finished
    _ppTransferLock.release()
    try:
##	import ZPublisher	# Maybe we can do this above in the modules-main-part, but I'm not sure if there'll be problems with multiple threads causing conflicts/dead-lock/invalid-data
	resultD= {}
	ZPublisher.Zope(ppHandlerLocation, s=1, u=userP+':'+passwdP, extra={'mutResult':resultD, 'pThreadID':ptID , 'accKey':keyD, 'accValue':valueD})
	wtD= float(str(resultD['result']))
	del resultD
	AccessControl.SecurityManagement.noSecurityManager()	# Whaaaa I've to clean up the SecurityManager manually, because this Thread is finished here and otherwise we'll get a memory-leak
##	del ZPublisher
    except:     # something went wrong, maybe ConflictError, so just wait a little and try again
	wtD= (random.random()*5) + 0.5
	sys.stderr.write('PThreads: triggering PPHandler this error occurred: ')	# Log this error
	traceback.print_exc(file=sys.stderr)
    _ppTransferLock.acquire()	# Remove this key
    if _accessKeysDict.has_key(keyD):
	del _accessKeysDict[keyD]
    if _manipulatorDict.has_key(ptID):
	del _manipulatorDict[ptID]
    _ppTransferLock.release()
    return wtD




def initPPHandler(prodP):
    try:
        ppHandlerD= getattr(prodP, ppHandler_action, None)
    except:
        ppHandlerD= None
    if ( (ppHandlerD is None) or (ppHandlerD != ppHandlerMethod) ):
        setattr(prodP, ppHandler_action, ppHandlerMethod)
	listD= []
	for eleR in prodP.manage_options:
	    if eleR.get('label', None) != ppHandler_label:
		listD.append(eleR)
	listD.append({'label':ppHandler_label, 'action':ppHandler_action,})
	prodP.manage_options= tuple(listD)
	prodP._p_changed= 1
	get_transaction().note('Added a "'+str(ppHandler_action)+'" to the "'+str(prodP)+'" Product')
	get_transaction().commit()



def ppHandlerMethod(self, mutResult={}, pThreadID=None, accKey=None, accValue=None, REQUEST=None):
    """ handle a request to invoke a PP """
    mutResult['result']= str(-1)	# This is the fallback-result
    pChkD= checkPermission(ppHandler_permission, self, REQUEST)		# Whaaa, Zope just didn't make it right
    if (pChkD is not None):
	mutResult['result']= str(pChkD)
	return str(pChkD)
    if ((pThreadID is None) and (accKey is None) and (accValue is None) and (REQUEST is not None)):
	return 'PThreads-Zope: The user "' + str(REQUEST.get('AUTHENTICATED_USER', '')) + '" can access the ppHandlerTrigger (see PersistentThreads-Help for more info).\n'
    pThreadID= str(pThreadID)
    accKey= str(accKey)
    accValue= str(accValue)
    _ppTransferLock.acquire()	# Do the Security Check
    if not _accessKeysDict.has_key(accKey):
        return str(-1)
    if (_accessKeysDict[accKey] != accValue):
        return str(-1)
    maniList= _manipulatorDict.get(pThreadID, None)
    _ppTransferLock.release()
    ppManD= PPManager._getPPMan(self)
    if ppManD is None:	# Oooops it's too early
	mutResult['result']= str(13.3)
	return '13.3'
    ppD= ppManD.getPP(pThreadID)
    if ppD is None:
	mutResult['result']= str(-1)
	return '-1'
    if maniList:
	try:
	    for maniR in maniList:
		if not maniR._completed:
		    maniR.manipulate(ppD, REQUEST=REQUEST)
		    maniR._completed= 1
	    resD= -2
	except:
	    resD= 0	# Manipulation not completed
	    import sys, traceback, string
	    type, val, tb = sys.exc_info()
	    sys.stderr.write(string.join(traceback.format_exception(type, val, tb), ''))
	    del type, val, tb
    else:
	resD= ppD.process(REQUEST=REQUEST)
    try:
	retD= float(resD)
    except:
	retD= -1
    if ((retD == -1) and ppD.autoBurry()):
	ppManD.delPP(pThreadID)
    mutResult['result']= str(retD)
    return str(retD)





