#____________________________________________________________________________________
#
#    PersistentThreads (see README.txt for more information)
#    Copyright (C) 2002 Christoph Becker-Freyseng
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#    christoph@mmc-startup.com
#    http://www.beyond-thoughts.com
#    http://www.mmc-startup.com
#____________________________________________________________________________________
#



__doc__="""Persistent Threads, PThreadMan is doing scheduling Voodoo"""
__version__='0.1.1'


import threading
from time import time
import random

from PPHandler import handlePPCall
import PTTools



def createAsyncLoop(startupFunc):	# Whaaaaa Zope hangs if I just start a Python-Thread before it's finally started (esp. when NUMBER_OF_THREADS=1)
    from ZServer import asyncore
    def asyncD(timeout=30.0, use_poll=0, map=None, startupFunc=startupFunc, oldLoop=asyncore.loop):
	startupFunc()
	oldLoop(timeout, use_poll, map)
    return asyncD



class PThreadMan(threading.Thread):
    _idleWaitTime= 11.2
    _waitInterval= 0.7
    _maxManipulationTries= 10		# How many times should PPHandler try to process the manipulators list (here we can't just try infinity)
    
    def __init__(self):
	threading.Thread.__init__(self, name='PThreadMan Thread')
	self.setDaemon(1)
	self._ppDict= {}		# Dict containing (user, password) as value and as key the threadID is used
	self._ptDict= {}		# Dict with the times (value) when the threads (key=ptID) want to run time.time()
	self._cond = threading.Condition()	# Make sure we don't mess up our data, make some shortcuts
	self._a= self._cond.acquire
	self._r= self._cond.release
	self._w= self._cond.wait
	self._n= self._cond.notify
	self._nA= self._cond.notifyAll
	self._runningDict= {}		# Dict that contains the PThreads-IDs actually running (key) and the thread that is controlling it (value)
	self._killerDict= {}		# Dict that contains the PThread-IDs somebody wants to kill and as value an Condition so you can wait till job's done
	self._manipulatorDict= {}	# Dict that contains the PThread-IDs somebody wants to manipulate its PP (key). The value a tuple: an Condition so you can wait till manipulators job's done, (user, password), List of Manipulators
	self.waitTime= 0.0		# general System-Time until the earliest Thread wants to run.
	from ZServer import asyncore
	asyncore.loop= createAsyncLoop(self.start)
    
    def start(self):
	if PTTools.getNumberOfThreads()<2:
	    import sys
	    sys.stderr.write('PersistentThreads Error: Your current NUMBER_OF_THREADS is only ' + str(PTTools.getNumberOfThreads()) + ' this is UNSTABLE (!!!!) with PersistentThreads. Won\'t start PT till fixed.\n')
	else:
	    threading.Thread.start(self)
    
    def run(self):
	while 1:
	    self._a()
	    # Calculate the time we can wait -- find out when the earliest Thread want's to run
	    if len(self._ptDict):
		self.waitTime= min(self._ptDict.values())
	    else:
		self.waitTime= self._idleWaitTime + time()	# Nothing to do, just wait a bit
	    for ptIDR in self._manipulatorDict.keys():
		self.handleManipulate(ptIDR)
	    if len(self._manipulatorDict):	# It's still not empty so manipulations are not complete -> don't wait
		self.waitTime= 0.0
	    if (self.waitTime > time()):	# So no Thread wants to run now -> wait.
		while (self.waitTime - time()) > self._waitInterval:
		    self._w(self._waitInterval)
		self._w(max(0, self.waitTime - time()))		# max(..) avoid waiting a negative amount
	    timeD= time()	# This is only for performance. If you think it's exact enough just comment it out and replace timeD with time() below
	    for (ptIDR, wtR) in self._ptDict.items():
	        if (wtR <= timeD):
		    self.handlePT(ptIDR)
	    self._r()
    
    def handleManipulate(self, ptID):
	if self._runningDict.has_key(ptID):	# make sure it won't run twice e.g. when it's running just now
	    self._runningDict.get(ptID).thinkAMoment()	# Say that it should interupt its running
	    return 0	# It's running already
	else:
	    mEntryD= self._manipulatorDict[ptID]
	    del self._manipulatorDict[ptID]	# Manipulation going to be done
	    hThreadD= PThreadManHandleThread(self.handlerManipulate, ptID, PThreadManHandleThread.MISSION_MANIPULATE, mEntryD)	# Maybe we're messing up gc here
	    self._runningDict[ptID]= hThreadD
	    hThreadD.start()
	    return 1
    
    def handlerManipulate(self, ptID, mEntryP):
	(mCond, (ppUser, ppPasswd), manipulatorList)= mEntryP
	mTimesD= 0
	while ((mTimesD<self._maxManipulationTries) and (handlePPCall(ptID, ppUser, ppPasswd, manipulatorList)!=-2)):
	    mTimesD=mTimesD+ 1
	    from time import sleep	# Something's wrong -> wait a bit
	    sleep(0.7)
	mCond.acquire()
	mCond.notifyAll()      # tell the agents, this PProcessor has been manipulated
	mCond.release()
	self._cond.acquire()
	del self._runningDict[ptID]
	self._cond.release()
    
    def handlePT(self, ptID):
	if self._runningDict.has_key(ptID):	# make sure it won't run twice e.g. when it's manipulated just now
	    self._ptDict[ptID]= 0.0	# Maybe it can't start now, because its PP is being manipulated -> make sure it's started as soon as possible
	    return 0	# It's running already
	else:
	    hThreadD= PThreadManHandleThread(self.handlerPT, ptID, PThreadManHandleThread.MISSION_RUN)
	    self._runningDict[ptID]= hThreadD
	    del self._ptDict[ptID]	# remove from the _ptDict for waiting PThreads, because this one is running. We still can keep track of it because it's registered in _runningDict now
	    hThreadD.start()
	    return 1
    
    def handlerPT(self, ptID):
	self._cond.acquire()
	(ppUser, ppPasswd)= self._ppDict[ptID]
	thisThread= self._runningDict[ptID]
	if self._killerDict.has_key(ptID):	# PThread's killed via Killer will become Zombie
	    waitTime= -1    # Kill him
	else:
	    waitTime= 0.0
	self._cond.release()
	while ((waitTime==0.0) and (thisThread.shouldThink()==0)):
	    waitTime= handlePPCall(ptID, ppUser, ppPasswd)
	self._cond.acquire()
	if self._killerDict.has_key(ptID):	# PThread's killed via Killer will become Zombie
	    waitTime= -1    # Kill him
	if waitTime>=0:
	    self._ptDict[ptID]= waitTime
	    if self.waitTime>waitTime:
		self.waitTime= waitTime
		self._cond.notifyAll()	# stop waiting
	else:	# it's dead
##	    del self._ptDict[ptID]	# This is done earlier now. see handlePT
	    del self._ppDict[ptID]
	    if self._killerDict.has_key(ptID):
		self._killerDict[ptID].acquire()
		self._killerDict[ptID].notifyAll()	# tell the agents, this PThread is killed
		self._killerDict[ptID].release()
		del self._killerDict[ptID]
	del self._runningDict[ptID]
	self._cond.release()
    
    def addPP(self, ptID, ppUser, ppPasswd):
	self._cond.acquire()
	self._ppDict[ptID]= (ppUser, ppPasswd)
	self._cond.release()
    
    def hasPP(self, ptID):
	self._cond.acquire()
	boolD= self._ppDict.has_key(ptID)
	self._cond.release()
	return boolD
    
    def startPP(self, ptID, delayP=60.3):
	self._cond.acquire()
	if self._ptDict.has_key(ptID):
	    self._cond.release()
	    return 0	# already started
	threadD= self._runningDict.get(ptID, None)
	if threadD is not None:
	    if threadD.getMission()==PThreadManHandleThread.MISSION_RUN:
		self._r()
		return 0	# already started
	self._ptDict[ptID]= delayP+time()	# make sure it won't start too early
	if self.waitTime>self._ptDict[ptID]:
	    self.waitTime= self._ptDict[ptID]
	self._cond.notifyAll()	# stop waiting
	self._cond.release()
	return 1
    
    def stopPP(self, ptID):
	self._cond.acquire()
##	if not self._ptDict.has_key(ptID):
##	    self._cond.release()
##	    return 0	# it was not started started
	threadD= self._runningDict.get(ptID, None)
	if not self._ptDict.has_key(ptID):
	    if threadD is None:
		self._r()
		return 0    # it was not started started
	    elif threadD.getMission()!=PThreadManHandleThread.MISSION_RUN:
		self._r()
		return 0    # it was not started started
	if self._killerDict.has_key(ptID):	# Someone else already tries to kill him
	    condD= self._killerDict[ptID]
	    self._cond.release()
            condD.acquire()
	else:	# rent a killer
	    condD= threading.Condition()
	    condD.acquire()
	    self._killerDict[ptID]= condD
	    if threadD is None:		# Killer can only kill a Thread when it works... so make it busy
		self._ptDict[ptID]= 0.0
		self.waitTime= 0.0
		self._cond.notifyAll()	# stop waiting
	    elif threadD.getMission()!=PThreadManHandleThread.MISSION_RUN:
		self._ptDict[ptID]= 0.0
		self.waitTime= 0.0
		self._cond.notifyAll()	# stop waiting
	    else:# It's running already. Say that it should interupt
		threadD.thinkAMoment()
	    self._cond.release()
	condD.wait()	# Wait until killers job is done
	condD.release()
	return 1
    
    def statusPP(self, ptID):	# see PPManager.statusPP
	self._cond.acquire()
	valD= self._ptDict.get(ptID, None)
	if valD is None:	# might be a Zombie or rnning etc. -> find out
	    retD= -666
	    threadD= self._runningDict.get(ptID, None)
	    if threadD is not None:
		if threadD.getMission()==PThreadManHandleThread.MISSION_RUN:
		    retD= 0
	elif valD == -1:
	    retD= -1
	else:
	    retD= valD
	self._cond.release()
	return retD
    
    def manipulatePP(self, ptID, ppUser, ppPasswd, manipulatorP):	# see PPManager.manipulatePP
	self._cond.acquire()
	if self._manipulatorDict.has_key(ptID):	# Someone else already tries to manipulate him
	    condD= self._manipulatorDict[ptID][0]
	    self._manipulatorDict[ptID][2].append(manipulatorP)	# Append to the List of Manipulators
	    self._cond.release()
            condD.acquire()
	else:	# Start the Manipulators-Crew
	    condD= threading.Condition()
	    condD.acquire()
	    self._manipulatorDict[ptID]= (condD, (ppUser, ppPasswd), [manipulatorP, ])
	    threadD= self._runningDict.get(ptID, None)
	    if threadD is not None:	# It's running already. Say that it should interupt
		threadD.thinkAMoment()
	    # PP can only get manipulated when the main-loop is re-run -> make it so
	    self.waitTime= 0.0
	    self._cond.notifyAll()	# stop waiting
	    self._cond.release()
	condD.wait()	# Wait until killers job is done
	condD.release()
	return 1





class PThreadManHandleThread(threading.Thread):
    MISSION_RUN= 1
    MISSION_MANIPULATE= 2
    
    def __init__(self, targetP, ptIDP, missionP, mEntryP=None):
	if mEntryP is None:
	    argsD=(ptIDP, )
	else:
	    argsD=(ptIDP, mEntryP, )
        threading.Thread.__init__(self, name='PThreadManHandleThread:'+str(ptIDP), target=targetP, args=argsD)
        self.setDaemon(1)
	self.mission= missionP
	self._lock= threading.Lock()
	self._think= 0
    
    def getMission(self):
	return self.mission
    
    def thinkAMoment(self):
	self._lock.acquire()
	self._think= 1
	self._lock.release()
    
    def shouldThink(self):
	self._lock.acquire()
	thinkD= self._think
	self._think= 0
	self._lock.release()
	return thinkD






