#____________________________________________________________________________________
#
#    PersistentThreads (see README.txt for more information)
#    Copyright (C) 2002 Christoph Becker-Freyseng
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#    christoph@mmc-startup.com
#    http://www.beyond-thoughts.com
#    http://www.mmc-startup.com
#____________________________________________________________________________________
#


__doc__="""A PersistentProcessor (PP) is the baseclass for all things that 
	    want to run as PersistentThread. Each PP will have a thread
	    that triggers it's process method.
	    A UI_PProcessor defines some additional methods, so that it can
	    be visually managed by PThreadTool. But there are many cases
	    where you don't need such a UI e.g. a Zope-based 
	    downloadmanager may use PThreads for downloading in background.
	    Then the dm will have an UI itself and allows the User to
	    start, stop, ... downloads hiding the PThreads-machinery."""
__version__='0.1.1'





import PPManager

import threading
import string
from Globals import Persistent
import Acquisition



class PersistentProcessor(Persistent, Acquisition.Implicit, PPManager.PPManUtilMixin):
    isA_PersistentProcessor= 1
    def __init__(self):
	self._mature= 0	# PPs are not mature until they were started a first time. This prevents the starting of not ready-configured PThreads when Zope restarts
    
    def process(self, REQUEST=None):
	""" This method processes the data and returns the time to wait until next call 
	    or -1 to signal dead """
	return -1  # UUaaa if unimplemented this will die soon
    
    def autoBurry(self): # burry when dead
	return 1
    






class UI_PProcessor(PersistentProcessor):
    isA_UI_PProcessor= 1
    def __init__(self):
	PersistentProcessor.__init__(self)
    
    def getUI_init(self, REQUEST):
	"""Return some HTML for init, process init_data, return None if OK or 
	HTML on error"""
	return None
    
    def getUI_manage(self, ptID, REQUEST):
	"""Return some HTML for management of this PP."""
	return ''






