#____________________________________________________________________________________
#
#    PersistentThreads (see README.txt for more information)
#    Copyright (C) 2002 Christoph Becker-Freyseng
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#    christoph@mmc-startup.com
#    http://www.beyond-thoughts.com
#    http://www.mmc-startup.com
#____________________________________________________________________________________
#


__doc__="""Some PPs that might be useful or at least an example."""
__version__='0.1.1'




import random, sys
import string
import os
import traceback
import time

import PersistentProcessor
import PPManipulator
import PTTools
import Products.PersistentThreads





class UniversalManipulator(PPManipulator.PPManipulator):
    def __init__(self, varDictP, getDictP):
	PPManipulator.PPManipulator.__init__(self)
	self.varDict= varDictP
	self.getDict= getDictP
    
    def manipulate(self, ppP, REQUEST=None):
	for (keyR,valueR) in self.varDict.items():
	    setattr(ppP, keyR, valueR)
	for keyR in self.getDict.keys():
	    self.getDict[keyR]= getattr(ppP, keyR)



class TestUIPP(PersistentProcessor.UI_PProcessor):
    
    def __init__(self):
	PersistentProcessor.UI_PProcessor.__init__(self)
	self.user= ''
	self.passwd= ''
	self.msg= 'Hello World'
	self.ttWait= 60.0
    
    def process(self, REQUEST=None):
	""" This method processes the data and returns the time to wait until next call 
	    or -1 to signal dead """
	sys.stderr.write(self.msg + '\n')  # stdout won't show ???
	return self.ttWait + time.time()
    
    def getUI_init(self, REQUEST):
	getDict= {'user':self.user, 'msg':self.msg, 'ttWait':self.ttWait}
	if REQUEST.has_key('ppUI_2stage'):
	    resD= self._doUpdate(None, getDict, REQUEST)
	    if resD:
		return (resD + self._inputForm(getDict))
	    else:
		return None
	else:
	    return self._inputForm(getDict)
    
    def getUI_manage(self, ptID, REQUEST):
	getDict= {'user':self.user, 'msg':self.msg, 'ttWait':self.ttWait}
	resD= self._doUpdate(ptID, getDict, REQUEST)
	return (resD + self._inputForm(getDict))
    
    def _doUpdate(self, ptID, getDict, REQUEST):
	strD= ''
	try:
	    userD= str(getattr(REQUEST, 'ppUI_user', self.user))
	    if not userD:
		raise 'User must not be empty'
	    passwdD= str(getattr(REQUEST, 'ppUI_passwd', self.passwd))
	    if not passwdD:
		raise 'Password must not be empty'
	    msgD= str(getattr(REQUEST, 'ppUI_msg', self.msg))
	    ttWaitD= float(getattr(REQUEST, 'ppUI_ttWait', self.ttWait))
	    if ptID is None:
		self.user= userD
		self.passwd= passwdD
		self.msg= msgD
		self.ttWait= ttWaitD
	    else:
		maniD= UniversalManipulator({'user':userD, 'passwd':passwdD, 'msg':msgD, 'ttWait':ttWaitD}, getDict)
		ppManD= self._getPPMan()
		ppManD.manipulatePP(ptID, maniD)
	except:
	    import sys, traceback, string
	    type, val, tb = sys.exc_info()
	    strD=strD+ '<h2>Error: ' + string.join(traceback.format_exception(type, val, tb), '') + '</h2>\n'
	    del type, val, tb
	return strD
    
    def _inputForm(self, getDict):
	strD= ''
        strD=strD+ '<b>User: </b><input type="text" name="ppUI_user" size=40 value="'+str(getDict['user'])+'"><br>'
        strD=strD+ '<b>Password: </b><input type="password" name="ppUI_passwd" size=40><br>'
	strD=strD+ '<b>Message: </b><input type="text" name="ppUI_msg" size=40 value="'+str(getDict['msg'])+'"><br>'
	strD=strD+ '<b>Time2Wait: </b><input type="text" name="ppUI_ttWait" size=40 value="'+str(getDict['ttWait'])+'"><br>'
	strD=strD+ '<input type="hidden" name="ppUI_2stage" value="yes">'
	return strD


PTTools.registerUI_PP(TestUIPP, 'my TestUIPP')










class Ring2PingUIPP(PersistentProcessor.UI_PProcessor):
    
    def __init__(self):
	PersistentProcessor.UI_PProcessor.__init__(self)
	self.user= ''
	self.passwd= ''
	self.msgLogFile= '/var/log/messages'
	self.phoneTag= 'Your Number'
	self.pingAddr= '141.1.1.1'
	self.checkFrq= 1.0
	self.pingTimes= 30
	self.pingFrq= 35
	self._v_logFile= None
	self._v_doPing= 0
    
    def __setstate__(self, state):
	""" initialize the file-object."""
        PersistentProcessor.UI_PProcessor.__setstate__(self, state)
	self._v_logFile= None
	self._v_doPing= 0
    
    def process(self, REQUEST=None):
	""" This method processes the data and returns the time to wait until next call 
	    or -1 to signal dead """
	if self._v_logFile is None:
	    try:
		self._v_logFile= open(self.msgLogFile, 'r')
		self._v_logFile.seek(0,2)	# This is the end
	    except:
		traceback.print_exc(file=sys.stderr)
		return -1
	for lineR in self._v_logFile.readlines():
	    if string.find(lineR, self.phoneTag)>=0:
		import sys
		sys.stderr.write('Starting ping.\n')
		self._v_doPing= 1
	if self._v_doPing > self.pingTimes:
	    self._v_doPing= 0
	if self._v_doPing:
	    os.system('/bin/ping -c 1 '+self.pingAddr+' > /dev/null 2> /dev/null')
	    self._v_doPing+= 1
	    return self.pingFrq + time.time()
	else:
	    return self.checkFrq + time.time()
    
    def autoBurry(self): # become zombie HAHAHAHAHAHAAAAAAA
	return 0
    
    def getUI_init(self, REQUEST):
	getDict= {
		    'msgLogFile':self.msgLogFile,
		    'phoneTag':self.phoneTag,
		    'pingAddr':self.pingAddr,
		    'checkFrq':self.checkFrq,
		    'pingTimes':self.pingTimes,
		    'pingFrq':self.pingFrq,
		}
	if REQUEST.has_key('ppUI_2stage'):
	    resD= self._doInitUpdate(REQUEST)
	    res2D= self._doUpdate(None, getDict, REQUEST)
	    if resD or res2D:
		return (resD + res2D + self._inputInitForm() + self._inputForm(getDict))
	    else:
		return None
	else:
	    return self._inputInitForm() + self._inputForm(getDict)
    
    def getUI_manage(self, ptID, REQUEST):
	getDict= {
		    'msgLogFile':self.msgLogFile,
		    'phoneTag':self.phoneTag,
		    'pingAddr':self.pingAddr,
		    'checkFrq':self.checkFrq,
		    'pingTimes':self.pingTimes,
		    'pingFrq':self.pingFrq,
		}
	resD= self._doUpdate(ptID, getDict, REQUEST)
	return (resD + self._inputForm(getDict))
    
    def _doInitUpdate(self, REQUEST):
	strD= ''
	try:
	    userD= str(getattr(REQUEST, 'ppUI_user', self.user))
	    if not userD:
		raise 'User must not be empty'
	    passwdD= str(getattr(REQUEST, 'ppUI_passwd', self.passwd))
	    if not passwdD:
		raise 'Password must not be empty'
	    self.user= userD
	    self.passwd= passwdD
	except:
	    import sys, traceback, string
	    type, val, tb = sys.exc_info()
	    strD=strD+ '<h2>Error: ' + string.join(traceback.format_exception(type, val, tb), '') + '</h2>\n'
	    del type, val, tb
	return strD
    
    def _inputInitForm(self):
	strD= ''
        strD=strD+ '<b>User: </b><input type="text" name="ppUI_user" size=40 value="'+str(self.user)+'"><br>'
        strD=strD+ '<b>Password: </b><input type="password" name="ppUI_passwd" size=40><br>'
	strD=strD+ '<input type="hidden" name="ppUI_2stage" value="yes">'
	return strD
    
    def _doUpdate(self, ptID, getDict, REQUEST):
	strD= ''
	try:
	    msgFD= str(getattr(REQUEST, 'ppUI_msgF', self.msgLogFile))
	    if not msgFD:
		raise 'MsgLogFile must not be empty'
	    phoneD= str(getattr(REQUEST, 'ppUI_phone', self.phoneTag))
	    if not phoneD:
		raise 'PhoneTag must not be empty'
	    pingAdD= str(getattr(REQUEST, 'ppUI_pingAd', self.pingAddr))
	    if not phoneD:
		raise 'Ping Address must not be empty'
	    checkFrqD= float(getattr(REQUEST, 'ppUI_checkFrq', self.checkFrq))
	    pingTimesD= float(getattr(REQUEST, 'ppUI_pingTimes', self.pingTimes))
	    pingFrqD= float(getattr(REQUEST, 'ppUI_pingFrq', self.pingFrq))
	    if ptID is None:
		self.msgLogFile= msgFD
		self.phoneTag= phoneD
		self.pingAddr= pingAdD
		self.checkFrq= checkFrqD
		self.pingTimes= pingTimesD
		self.pingFrq= pingFrqD
	    else:
		maniD= UniversalManipulator({
		    'msgLogFile': msgFD,
		    'phoneTag': phoneD,
		    'pingAddr': pingAdD,
		    'checkFrq': checkFrqD,
		    'pingTimes': pingTimesD,
		    'pingFrq': pingFrqD,
		}, getDict)
		ppManD= self._getPPMan()
		ppManD.manipulatePP(ptID, maniD)
	except:
	    import sys, traceback, string
	    type, val, tb = sys.exc_info()
	    strD=strD+ '<h2>Error: ' + string.join(traceback.format_exception(type, val, tb), '') + '</h2>\n'
	    del type, val, tb
	return strD
    
    def _inputForm(self, getDict):
	strD= ''
        strD=strD+ '<b>MsgLogFile: </b><input type="text" name="ppUI_msgF" size=40 value="'+str(getDict['msgLogFile'])+'"><br>'
        strD=strD+ '<b>PhoneTag: </b><input type="text" name="ppUI_phone" size=40 value="'+str(getDict['phoneTag'])+'"><br>'
	strD=strD+ '<b>Ping Address: </b><input type="text" name="ppUI_pingAd" size=40 value="'+str(getDict['pingAddr'])+'"><br>'
	strD=strD+ '<b>Check Frq: </b><input type="text" name="ppUI_checkFrq" size=40 value="'+str(getDict['checkFrq'])+'"><br>'
	strD=strD+ '<b>Ping Times: </b><input type="text" name="ppUI_pingTimes" size=40 value="'+str(getDict['pingTimes'])+'"><br>'
	strD=strD+ '<b>Ping Frq: </b><input type="text" name="ppUI_pingFrq" size=40 value="'+str(getDict['pingFrq'])+'"><br>'
	return strD


PTTools.registerUI_PP(Ring2PingUIPP, 'Ring2Ping')






